//+------------------------------------------------------------------+
//|                                         WriteTicksFromTester.mq5 |
//|                        Copyright 2011, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2011, MetaQuotes Software Corp."
#property link      "http://www.mql5.com"
#property version   "1.00"
//+------------------------------------------------------------------+
//|                                                 |
//+------------------------------------------------------------------+
enum Mode
  {
   everytick,    //  
   ohlc,         // OHLC
   openprice,    //   
  };
//--- input parameters
input datetime start=D'2011.01.11 21:00:00';
input datetime end=D'2011.01.11 21:30:00';
input string   filenameEveryTick="everytick.csv";
input string   filenameOHLC="ohlc.csv";
input string   filenameOpenPrice="openprice.csv";

Mode   type=everytick;    //  
int    file;
string filename;
int    minutes_counter;
int    tick_counter;
//+------------------------------------------------------------------+
//|       /                  |
//+------------------------------------------------------------------+
struct PriceTime
  {
   datetime          time;
   double            price;
   //--- 
                     PriceTime() {time=0;price=0;};
  };

PriceTime ticks_array[];
//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
  {
   Print(__FUNCTION__);
//---  
   minutes_counter=0;
   tick_counter=0;
//---      PriceTime
   ArrayResize(ticks_array,1000,100);
   Print("Old version "+__FILE__);
//---
   return(0);
  }
//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
//---
   PrintFormat(": %d,  : %d",tick_counter,minutes_counter);

//---     
   double rate=double(tick_counter)/double(minutes_counter);
   int d_rate=(int)MathRound(rate);
   switch(d_rate)
     {
      case 1:
        {
         filename=filenameOpenPrice;
         break;
        }
      case 4:
        {
         filename=filenameOHLC;
         break;
        }
      default: filename=filenameEveryTick;
     }
//---   
   filename=_Symbol+"_"+EnumToString(_Period)+"_"+filename;
   Print("     "+filename);

//---  
   ResetLastError();
//---  
   file=FileOpen(filename,FILE_WRITE|FILE_CSV|FILE_COMMON,";");
//---   
   if(file==INVALID_HANDLE)
     {
      PrintFormat("      %s.  =%d",filename,GetLastError());
      return;
     }
   else
     {
      PrintFormat("     %s",TerminalInfoString(TERMINAL_COMMONDATA_PATH));
     }
//---    
   for(int i=0;i<tick_counter;i++)
     {
      FileWrite(file,long(ticks_array[i].time),ticks_array[i].price,TimeToString(ticks_array[i].time,TIME_DATE|TIME_SECONDS));
     }
//---  
   FileClose(file);
  }
//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick()
  {
//Print(__FUNCTION__);
   datetime curr_time=TimeCurrent();
   if(curr_time<start || curr_time>end) return;
//---    
   MqlTick tick;
   SymbolInfoTick(_Symbol,tick);
   CollectTicks(ticks_array,tick,minutes_counter,tick_counter);
  }
//+------------------------------------------------------------------+
//|                                                 |
//+------------------------------------------------------------------+
void CollectTicks(PriceTime &array[], //  
                  MqlTick &tick,      //   
                  int &minute_counter,//  
                  int &counter        //   ( )
                  )
  {
//Print(tick.time);
//---       
   static int last_min=-1;
//---    
   int size=ArraySize(array);
//---      
   if(counter>=size-1)
     {
      //---      100 
      ArrayResize(array,size+100,100);
     }
//---      
   array[counter].price=tick.bid;
   array[counter].time=tick.time;
//---  
   counter++;

//---    
   MqlDateTime  dt_struct;
   TimeToStruct(tick.time,dt_struct);
//---      
   int min=dt_struct.min;
//---      ,    
   if(last_min!=min)
     {
      //---   
      last_min=min;
      //---  
      minute_counter++;
      //PrintFormat("    %s",TimeToString(tick.time,TIME_DATE|TIME_SECONDS));
     }
//---
  }
//+------------------------------------------------------------------+
